var name = "@jitsu/pipeline-helpers";
var version = "0.5.0";
var description = "Utility functions designed to outsource some processing logic from Server to JS";
var author = "Jitsu Dev Team <dev@jitsu.com>";
var publishConfig = {
	access: "public"
};
var homepage = "https://github.com/jitsucom/jitsu-sdk#readme";
var license = "MIT";
var main = "lib/index.js";
var module = "lib/index.es.js";
var types = "lib/index.d.ts";
var files = [
	"lib/*"
];
var repository = {
	type: "git",
	url: "https://github.com/jitsucom/jitsu-sdk",
	directory: "packages/pipeline-helpers"
};
var scripts = {
	build: "rm -rf ./lib && rollup -c && mv ./lib/src/*.d.ts ./lib && rmdir ./lib/src",
	test: "jest"
};
var bugs = {
	url: "https://github.com/jitsucom/jitsu/issues"
};
var devDependencies = {
	"@rollup/plugin-json": "^4.1.0",
	"@segment/analytics-next": "^1.32.1",
	"@types/jquery": "^3.5.10"
};
var gitHead = "dead29f98059ecf21a0576b0c64cdb7f71da46f7";
var pkg = {
	name: name,
	version: version,
	description: description,
	author: author,
	publishConfig: publishConfig,
	homepage: homepage,
	license: license,
	main: main,
	module: module,
	types: types,
	files: files,
	repository: repository,
	scripts: scripts,
	bugs: bugs,
	devDependencies: devDependencies,
	gitHead: gitHead
};

const segmentEventsTypes = {
    alias: true,
    group: true,
    identify: true,
    page: true,
    screen: true,
    track: true,
};
const getSegmentType = (jitsuType, customMapping, defaultSegmentType) => {
    if (jitsuType === "user_identity" || jitsuType === "identify") {
        return "identify";
    }
    else if (jitsuType === "page" || jitsuType === "pageview") {
        return "page";
    }
    else if (segmentEventsTypes[jitsuType] !== undefined) {
        return jitsuType;
    }
    else {
        const customType = customMapping[jitsuType];
        return customType ?? defaultSegmentType;
    }
};
function removeProps(ev, props) {
    const copy = { ...ev };
    props.forEach(p => delete copy[p]);
    return copy;
}
const jitsuToSegment = (ev, { defaultSegmentType = "track", eventTypeMapping = {} } = {}) => {
    if (ev.src_payload && ev.src === "ajs") {
        return ev.src_payload;
    }
    let segmentEvent = {
        type: getSegmentType(ev.event_type, eventTypeMapping, defaultSegmentType),
        _metadata: {},
        anonymousId: ev.user?.anonymous_id,
        context: {
            ip: ev.source_ip,
            locale: ev.user_language,
            userAgent: ev.user_agent,
            page: {
                path: ev.doc_path,
                referrer: ev.referer,
                search: ev.doc_search,
                title: ev.page_title,
                url: ev.url,
            },
            campaign: {
                name: ev.utm?.name,
                term: ev.utm?.term,
                source: ev.utm?.source,
                medium: ev.utm?.medium,
                content: ev.utm?.content,
            },
        },
        event: ev.event_type,
        groupId: undefined,
        integrations: undefined,
        messageId: ev.event_id,
        name: "",
        properties: flatten(removeProps(ev, [
            "user",
            "source_ip",
            "user_language",
            "user_agent",
            "doc_path",
            "referer",
            "doc_search",
            "page_title",
            "url",
            "event_type",
            "event_id",
            "utc_time",
            "_timestamp",
            "eventn_ctx_event_id",
            "utm",
            "parsed_ua",
            "location",
            "eventn_ctx",
            "api_key",
            "app",
            "doc_encoding",
            "doc_host",
            "ids",
            "local_tz_offset",
            "screen_resolution",
            "src",
            "vp_size",
            "src_payload",
        ]), { skipArrays: true }),
        sentAt: ev.utc_time ? new Date(ev.utc_time) : new Date(),
        timestamp: ev.utc_time ? new Date(ev.utc_time) : new Date(),
        traits: flatten(removeProps(ev.user || {}, ["anonymous_id", "id"])),
        userId: ev.user?.id || ev.user?.email,
    };
    removeEmptyFields(segmentEvent);
    return segmentEvent;
};
function safeToString(obj) {
    if (obj === undefined || obj === null) {
        return undefined;
    }
    else if (typeof obj === "string") {
        return obj;
    }
    else if (typeof obj === "object") {
        return JSON.stringify(obj);
    }
    else {
        return obj + "";
    }
}
const segmentTables = {
    track: "tracks",
    page: "pages",
    identify: "identifies",
    group: "groups",
    alias: "aliases",
    screen: "screens",
};
const segmentToTable = (event, jitsuEvent) => {
    const payloadObj = jitsuEvent?.src_payload?.obj || {};
    return {
        JITSU_TABLE_NAME: segmentTables[event.type] || "tracks",
        __sql_type_timestamp: "timestamp",
        __sql_type_sent_at: "timestamp",
        anonymous_id: safeToString(event.anonymousId) || jitsuEvent?.src_payload?.anonymousId,
        context_campaign_source: event.context?.campaign?.source,
        context_ip: event.context?.ip,
        context_library_name: jitsuEvent?.context?.library?.version || "jitsu-bridge",
        context_library_version: pkg.version ,
        context_locale: event.context?.locale || payloadObj.context?.locale,
        context_page_path: event.context?.page?.path,
        context_page_referrer: event.context?.page?.referrer,
        context_page_search: event.context?.page?.search,
        context_page_title: event.context?.page?.title,
        context_page_url: event.context?.page?.url,
        context_user_agent: event.context?.userAgent,
        context_utm_source: event.context?.campaign?.source,
        email: safeToString(event.traits?.email),
        id: event.messageId,
        name: safeToString(event.traits?.name),
        path: event.context?.page?.path,
        referrer: event.context?.page?.referrer,
        search: event.context?.page?.search,
        sent_at: safeToString(event.sentAt),
        timestamp: payloadObj.timestamp || safeToString(event.timestamp),
        title: event.context?.page?.title,
        url: event.context?.page?.url,
        user_id: safeToString(event.userId) || jitsuEvent?.src_payload?.userId,
        ...(event.properties || {}),
    };
};
function canonicalSqlTypeHint(hint) {
    if (typeof hint === "object") {
        if (!Array.isArray(hint)) {
            return hint;
        }
        else {
            return { defaultCastType: "hint[0]", defaultDDLType: "hint[1]", dialects: {} };
        }
    }
    else {
        return { defaultCastType: hint, defaultDDLType: hint, dialects: {} };
    }
}
function removeEmptyFields(obj) {
    for (const [key, val] of Object.entries(obj)) {
        if (val !== null && typeof val === "object" && !Array.isArray(val)) {
            removeEmptyFields(val);
        }
        else if (val === undefined) {
            delete obj[key];
        }
    }
}
function flatten(obj, { separator = "_", skipArrays = false } = {}, path = []) {
    if (typeof obj !== "object") {
        throw new Error(`Can't flatten an object, expected object, but got" ${typeof obj}`);
    }
    if (obj === null) {
        throw new Error(`Can't flatten null value`);
    }
    if (Array.isArray(obj)) {
        if (!skipArrays) {
            throw new Error(`Can't flatten array`);
        }
        else {
            return {};
        }
    }
    const res = {};
    for (const [key, value] of Object.entries(obj)) {
        if (typeof value === "object") {
            Object.entries(flatten(value, { separator, skipArrays }, [...path, key])).forEach(([subKey, subValue]) => (res[key + separator + subKey] = subValue));
        }
        else if (typeof value == "function") {
            throw new Error(`Can't flatten object with function as a value of ${key}. Path to node: ${path.join(".")}`);
        }
        else {
            res[key] = value;
        }
    }
    return res;
}

export { canonicalSqlTypeHint, flatten, getSegmentType, jitsuToSegment, removeEmptyFields, removeProps, segmentEventsTypes, segmentToTable };
